"""
Add table for status changes

Revision ID: fcc7f97c58cc
Revises: 4cd21fff2b64
Create Date: 2018-07-04 10:50:19.567558

"""
from alembic import op
import sqlalchemy as sa
from sqlalchemy.dialects import postgresql

# revision identifiers, used by Alembic.
revision = 'fcc7f97c58cc'
down_revision = '4cd21fff2b64'
branch_labels = None
depends_on = None


def upgrade():
    op.create_unique_constraint(op.f('uq_assignment_task_id'), 'assignment',
                                ['task_id', 'assignment_id'])

    op.create_table('assignment_status_change',
                    sa.Column('id', postgresql.UUID(as_uuid=True), nullable=False),
                    sa.Column('created', sa.DateTime(), nullable=False),
                    sa.Column('occurred', sa.DateTime(), nullable=False),
                    sa.Column('assignment_id', postgresql.UUID(as_uuid=True), nullable=False),
                    sa.Column('status', postgresql.ENUM('Abandoned', 'Returned', 'Accepted',
                                                        'Submitted', 'Approved', 'Rejected',
                                                        name='assignment_status_enum',
                                                        create_type=False), nullable=False),
                    sa.ForeignKeyConstraint(['assignment_id'], ['assignment.id'],
                                            name=op.f('fk_assignment_status_change_assignment_id_assignment')),  # noqa
                    sa.PrimaryKeyConstraint('id', name=op.f('pk_assignment_status_change')),
                    sa.UniqueConstraint('assignment_id', 'status',
                                        name=op.f('uq_assignment_status_change_assignment_id')))

    op.execute("""
        CREATE OR REPLACE FUNCTION update_assignment_status()
        RETURNS TRIGGER AS
        $BODY$
        BEGIN
        UPDATE assignment SET status = (
             SELECT status FROM assignment_status_change
             WHERE assignment_id = NEW.assignment_id
             ORDER BY occurred DESC LIMIT 1)
        WHERE id = NEW.assignment_id;
        RETURN NEW;
        END;
        $BODY$
        LANGUAGE plpgsql VOLATILE;

        CREATE TRIGGER update_assignment_status
        AFTER INSERT OR UPDATE ON assignment_status_change
        FOR EACH ROW EXECUTE PROCEDURE update_assignment_status();""")

    op.execute("""INSERT INTO assignment_status_change (id, created, occurred, assignment_id, status)
               (SELECT uuid_generate_v4(), now(), abandoned_time AS occurred, id, 'Abandoned'
                FROM assignment
                WHERE abandoned_time IS NOT NULL);""")
    op.drop_column('assignment', 'abandoned_time')

    op.execute("""INSERT INTO assignment_status_change (id, created, occurred, assignment_id, status)
               (SELECT uuid_generate_v4(), now(), returned_time AS occurred, id, 'Returned'
                FROM assignment
                WHERE returned_time IS NOT NULL);""")
    op.drop_column('assignment', 'returned_time')

    op.execute("""INSERT INTO assignment_status_change (id, created, occurred, assignment_id, status)
               (SELECT uuid_generate_v4(), now(), approved_time AS occurred, id, 'Approved'
                FROM assignment
                WHERE approved_time IS NOT NULL);""")
    op.drop_column('assignment', 'approved_time')

    op.execute("""INSERT INTO assignment_status_change (id, created, occurred, assignment_id, status)
               (SELECT uuid_generate_v4(), now(), submitted_time AS occurred, id, 'Submitted'
                FROM assignment
                WHERE submitted_time IS NOT NULL);""")
    op.drop_column('assignment', 'submitted_time')

    op.execute("""INSERT INTO assignment_status_change (id, created, occurred, assignment_id, status)
               (SELECT uuid_generate_v4(), now(), rejected_time AS occurred, id, 'Rejected'
                FROM assignment
                WHERE rejected_time IS NOT NULL);""")
    op.drop_column('assignment', 'rejected_time')

    op.execute("""INSERT INTO assignment_status_change (id, created, occurred, assignment_id, status)
               (SELECT uuid_generate_v4(), now(), accepted_time AS occurred, id, 'Accepted'
                FROM assignment
                WHERE accepted_time IS NOT NULL);""")
    op.drop_column('assignment', 'accepted_time')


def downgrade():
    pass
